﻿using System;
using System.IO;
using System.Reflection;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using C1.Silverlight.FlexGrid;

namespace C1.Silverlight.FlexGridExcel
{
    /// <summary>
    /// Specifies commands that may be executed from the context menu.
    /// </summary>
    public enum ContextMenuCommands
    {
        /// <summary>
        /// Cut the selection to the clipboard.
        /// </summary>
        Cut,
        /// <summary>
        /// Copy the selection to the clipboard.
        /// </summary>
        Copy,
        /// <summary>
        /// Paste the contents of the selection over the current selection.
        /// </summary>
        Paste,
        /// <summary>
        /// Clear the content of the selection without copying it to the clipboard.
        /// </summary>
        Clear,
        /// <summary>
        /// Insert new rows at the cursor.
        /// </summary>
        InsertRows,
        /// <summary>
        /// Delete the selected rows.
        /// </summary>
        DeleteRows,
        /// <summary>
        /// Insert new columns at the cursor.
        /// </summary>
        InsertColumns,
        /// <summary>
        /// Delete the selected columns.
        /// </summary>
        DeleteColumns
    }

    /// <summary>
    /// Context menu customized for use with the C1FlexGridExcel control.
    /// </summary>
    public class ExcelContextMenu : ContextMenu
    {
        //---------------------------------------------------------------------------
        #region ** fields

        C1FlexGridExcel _grid;
        Point _ptMouseMove;
        UIElement _root;
        static ContextMenu _lastOpenMenu;

        #endregion

        //---------------------------------------------------------------------------
        #region ** ctor

        /// <summary>
        /// Initializes a new instance of an <see cref="ExcelContextMenu"/>.
        /// </summary>
        /// <param name="grid"><see cref="C1FlexGridExcel"/> that owns this context menu.</param>
        internal ExcelContextMenu(C1FlexGridExcel grid)
        {
            // save reference to parent grid
            _grid = grid;

            // build menu
            Reset();

            // set up event handlers for context menu
            _grid.MouseMove += _grid_MouseMove;
            _grid.MouseRightButtonDown += _grid_MouseRightButtonDown;
        }

        #endregion

        //---------------------------------------------------------------------------
        #region ** object model
        
        /// <summary>
        /// Re-initializes the menu to its default content.
        /// </summary>
        void Reset()
        {
            // disconnect existing items
            foreach (var item in this.Items)
            {
                var mi = item as MenuItem;
                if (mi != null)
                {
                    mi.Click -= menuItem_Click;
                }
            }

            // remove all existing items
            Items.Clear();

            // populate the menu
            CreateMenuItem("Cut", ContextMenuCommands.Cut);
            CreateMenuItem("Copy", ContextMenuCommands.Copy);
            CreateMenuItem("Paste", ContextMenuCommands.Paste);
            Items.Add(new Separator());
            CreateMenuItem("Insert Rows", ContextMenuCommands.InsertRows);
            CreateMenuItem("Delete Rows", ContextMenuCommands.DeleteRows);
            CreateMenuItem("Insert Columns", ContextMenuCommands.InsertColumns);
            CreateMenuItem("Delete Columns", ContextMenuCommands.DeleteColumns);
        }
        /// <summary>
        /// Get a <see cref="MenuItem"/> based on a command code.
        /// </summary>
        /// <param name="cmd"><see cref="ContextMenuCommands"/> that determines the item to be retrieved.</param>
        /// <returns><see cref="MenuItem"/> that matches the given <paramref name="cmd"/>.</returns>
        public MenuItem GetMenuItem(ContextMenuCommands cmd)
        {
            foreach (var item in this.Items)
            {
                var mi = item as MenuItem;
                if (mi != null && object.Equals(mi.Tag, cmd))
                {
                    return mi;
                }
            }
            return null;
        }
        /// <summary>
        /// Executes a context menu command.
        /// </summary>
        /// <param name="cmd"><see cref="ContextMenuCommands"/> that specifies which 
        /// command should be executed.</param>
        public void ExecuteCommand(ContextMenuCommands cmd)
        {
            // save selection
            var sel = _grid.Selection;

            // execute command
            switch (cmd)
            {
                case ContextMenuCommands.Cut:
                    _grid.Copy();
                    _grid.ClearSelection();
                    break;

                case ContextMenuCommands.Copy:
                    _grid.Copy();
                    break;

                case ContextMenuCommands.Paste:
                    _grid.Paste();
                    break;

                case ContextMenuCommands.Clear:
                    _grid.ClearSelection();
                    break;

                case ContextMenuCommands.InsertRows:
                    // REVIEW: inserts unbound rows. If the grid is bound, this should 
                    // be disabled or modified to insert items in the data source.
                    sel = RemoveHeaderRow(sel);
                    if (sel.IsValid)
                    {
                        for (int i = 0; i < sel.RowSpan; i++)
                        {
                            _grid.Rows.Insert(sel.TopRow, new Row());
                        }
                    }
                    break;

                case ContextMenuCommands.DeleteRows:
                    // REVIEW: deletes the rows. If the grid is bound, this should 
                    // be disabled or modified to remove items from the data source.
                    sel = RemoveHeaderRow(sel);
                    if (sel.IsValid)
                    {
                        for (int i = 0; i < sel.RowSpan; i++)
                        {
                            _grid.Rows.RemoveAt(sel.TopRow);
                        }
                    }
                    break;

                case ContextMenuCommands.InsertColumns:
                    for (int i = 0; i < sel.ColumnSpan; i++)
                    {
                        _grid.Columns.Insert(sel.LeftColumn, new Column());
                    }
                    break;

                case ContextMenuCommands.DeleteColumns:
                    for (int i = 0; i < sel.ColumnSpan; i++)
                    {
                        _grid.Columns.RemoveAt(sel.LeftColumn);
                    }
                    break;
            }

            // restore selection
            try
            {
                _grid.Selection = sel;
            }
            catch
            {
                // old selection became invalid after deleting rows/columns
            }
        }
        /// <summary>
        /// Gets a reference to the grid that owns this menu.
        /// </summary>
        public C1FlexGrid OwnerGrid
        {
            get { return _grid; }
        }
        #endregion

        //---------------------------------------------------------------------------
        #region ** event handlers

        // save mouse coordinates to update selection when the context menu pops up
        void _grid_MouseMove(object sender, MouseEventArgs e)
        {
            _ptMouseMove = e.GetPosition(null);
        }
        void _grid_MouseRightButtonDown(object sender, MouseButtonEventArgs e)
        {
            // don't show Silverlight's built-in menu
            e.Handled = true;

            // close menu if open
            IsOpen = false;
            if (_lastOpenMenu != null && _lastOpenMenu.IsOpen)
            {
                _lastOpenMenu.IsOpen = false;
            }

            // get focus and update selection
            Focus();
            var ht = _grid.HitTest(e);
            if (!_grid.Selection.Contains(ht.CellRange))
            {
                _grid.Selection = ht.CellRange;
            }

            // adjust offset to account for bug in ContextMenu class
            var ptMouseDown = e.GetPosition(null);
            this.HorizontalOffset = ptMouseDown.X - _ptMouseMove.X;
            this.VerticalOffset = ptMouseDown.Y - _ptMouseMove.Y;

            // show menu
            _lastOpenMenu = this;
            IsOpen = true;

            // set parent's background to null so the grid can still get clicks
            var canvas = this.Parent as Canvas;
            if (canvas != null)
            {
                canvas.Background = null;
            }
        }
        /// <summary>
        /// Overridden to disable paste command if the clipboard is empty when the context menu opens.
        /// </summary>
        /// <param name="e"><see cref="RoutedEventArgs"/> that provides the event data.</param>
        protected override void OnOpened(RoutedEventArgs e)
        {
            // close the menu when the mouse goes down anywhere
            if (_root == null)
            {
                _root = Application.Current.RootVisual;
                if (_root != null)
                {
                    _root.AddHandler(UIElement.MouseLeftButtonDownEvent, new MouseButtonEventHandler(_root_MouseDown), true);
                }
            }

            // enable/disable Paste command
            var pasteItem = GetMenuItem(ContextMenuCommands.Paste);
            if (pasteItem != null)
            {
                pasteItem.IsEnabled = System.Windows.Clipboard.ContainsText();
            }

            // fire event as usual
            base.OnOpened(e);
        }
        void _root_MouseDown(object sender, MouseEventArgs e)
        {
            IsOpen = false;
        }

        #endregion

        //---------------------------------------------------------------------------
        #region ** implementation

        // remove header row from selection
        CellRange RemoveHeaderRow(CellRange sel)
        {
            if (_grid.Rows[sel.TopRow] is HeaderRow)
            {
                sel = new CellRange(sel.TopRow + 1, sel.LeftColumn, sel.BottomRow, sel.RightColumn);
            }
            if (_grid.Rows[sel.TopRow] is HeaderRow)
            {
                // only the header row was selected, so make the range invalid
                sel = CellRange.Empty;
            }
            return sel;
        }

        // create a menu item
        void CreateMenuItem(string hdr, ContextMenuCommands cmd)
        {
            // create item, attach event handler
            var mi = new C1MenuItem();
            mi.Header = hdr;
            mi.Tag = cmd;
            mi.Click += menuItem_Click;

            // create image for this item
            var resName = string.Format("{0}_small.png", cmd.ToString());
            var stream = GetResourceStream(resName);
            if (stream != null)
            {
                var bmp = new BitmapImage();
                bmp.SetSource(stream);
                var img = new Image();
                img.Source = bmp;
                mi.Icon = img;
            }

            // done
            Items.Add(mi);
        }

        // handle clicks on the menu items
        void menuItem_Click(object sender, RoutedEventArgs e)
        {
            var mi = sender as MenuItem;
            if (mi.Tag is ContextMenuCommands)
            {
                ExecuteCommand((ContextMenuCommands)mi.Tag);
            }
        }

        // get an embedded resource stream
        Stream GetResourceStream(string resName)
        {
            var asm = Assembly.GetExecutingAssembly();
            foreach (var rn in asm.GetManifestResourceNames())
            {
                if (rn.EndsWith(resName))
                {
                    return asm.GetManifestResourceStream(rn);
                }
            }
            return null;
        }

        #endregion
    }
}
