﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Xml.Linq;
using C1.C1Zip;
using C1.Silverlight.FlexGrid;
using C1.Silverlight.Pdf;

namespace C1.Silverlight.FlexGridExcel
{
    /// <summary>
    /// Provides PDF and printing options.
    /// </summary>
    public class PdfExportOptions
    {
        /// <summary>
        /// Initializes a new instance of a <see cref="PdfExportOptions"/>.
        /// </summary>
        public PdfExportOptions()
        {
            Margin = new Thickness(96);
            ScaleMode = ScaleMode.PageWidth;
            DocumentTitle = "ComponentOne FlexGrid";
        }
        /// <summary>
        /// Gets or sets the margins to use when printing or exporting a page.
        /// </summary>
        /// <remarks>
        /// Units are given in logical pixels (1/96").
        /// </remarks>
        public Thickness Margin { get; set; }
        /// <summary>
        /// Gets or sets a value that determines how the content should be scaled to
        /// fit on the document pages.
        /// </summary>
        public ScaleMode ScaleMode { get; set; }
        /// <summary>
        /// Gets or sets the document title.
        /// </summary>
        public string DocumentTitle { get; set; }
        /// <summary>
        /// Gets or sets whether the document should be printed in landscape mode.
        /// </summary>
        public bool Landscape { get; set; }
    }
    /// <summary>
    /// Provides utilities for exporting the grid to XLSX and PDF formats.
    /// </summary>
    internal static class GridExport
    {
        const string TEMPLATE_FILE_NAME = "template.zip";
        const string SHEET_NAME = "sheet1.xml";

        //-------------------------------------------------------------------------------------------------------
        #region ** object model

        public static void SavePdf(C1FlexGrid flex, Stream s)
        {
            SavePdf(flex, s, null);
        }
        public static void SavePdf(C1FlexGrid flex, Stream s, PdfExportOptions options)
        {
            // get rendering options
            if (options == null)
            {
                options = new PdfExportOptions();
            }

            // get root element to lay out the PDF pages
            Panel root = null;
            for (var parent = flex.Parent as FrameworkElement; parent != null; parent = parent.Parent as FrameworkElement)
            {
                if (parent is Panel)
                {
                    root = parent as Panel;
                }
            }

            // create pdf document
            var pdf = new C1PdfDocument(PaperKind.Letter, options.Landscape);

            // get page size
            var rc = pdf.PageRectangle;

            // create panel to hold elements while they render
            var pageTemplate = new PageTemplate();
            pageTemplate.Width = rc.Width;
            pageTemplate.Height = rc.Height;
            pageTemplate.SetPageMargin(options.Margin);
            root.Children.Add(pageTemplate);

            // render grid into PDF document
            var m = options.Margin;
            var sz = new Size(rc.Width - m.Left - m.Right, rc.Height - m.Top - m.Bottom);
            var pages = flex.GetPageImages(options.ScaleMode, sz, 100);
            for (int i = 0; i < pages.Count; i++)
            {
                // skip a page when necessary
                if (i > 0)
                {
                    pdf.NewPage();
                }

                // set content
                pageTemplate.PageContent.Child = pages[i];
                pageTemplate.PageContent.Stretch = System.Windows.Media.Stretch.Uniform;

                // set header/footer text
                pageTemplate.HeaderLeft.Text = options.DocumentTitle;
                pageTemplate.FooterRight.Text = string.Format("Page {0} of {1}",
                    i + 1, pages.Count);

                // measure page element
                pageTemplate.Measure(new Size(rc.Width, rc.Height));
                pageTemplate.UpdateLayout();

                // add page element to PDF
                pdf.DrawElement(pageTemplate, rc);
            }

            // done with template
            root.Children.Remove(pageTemplate);

            // save the PDF document
            pdf.Save(s);
            s.Close();
        }
        public static void SaveXlsx(C1FlexGrid flex, Stream s)
        {
            ExcelEngine.SaveXlsxFile(flex, s);
        }
        public static void SaveCsv(C1FlexGrid flex, Stream s)
        {
            ExcelEngine.SaveCsvFile(flex, s);
        }
        #endregion
    }
}
