﻿using System;
using System.Windows;
using System.Windows.Input;
using System.Collections;
using InvExplorer.ServiceReference1;

namespace InvExplorer
{
    /// <summary>
    /// Represents a command that removes selected portfolios from the portfolio collection.
    /// </summary>
    public class RemovePortfolioCommand : ICommand
    {
        ViewModel _model;

        /// <summary>
        /// Initializes a new instance of an <see cref="AddAssetCommand"/>.
        /// </summary>
        /// <param name="model"><see cref="ViewModel"/> that owns this command.</param>
        public RemovePortfolioCommand(ViewModel model)
        {
            _model = model;
            _model.PortfoliosView.CollectionChanged += (s, e) =>
            {
                OnCanExecuteChanged(EventArgs.Empty);
            };
        }
        /// <summary>
        /// Checks whether the command can be executed in the current model state,
        /// with the given parameter.
        /// </summary>
        /// <param name="parameter">Collection of <see cref="PortfolioItem"/> objects to 
        /// remove from the current portfolio.</param>
        /// <returns>True if the command can be executed with the given parameter.</returns>
        public bool CanExecute(object parameter)
        {
            // need a current portfolio
            if (_model.PortfoliosView == null)
            {
                return false;
            }

            // can't remove if we only have one portfolio
            if (_model.Portfolios.Count < 2)
            {
                return false;
            }

            // count items we can remove
            var cnt = 0;
            var items = parameter as IEnumerable;
            if (items != null)
            {
                foreach (Portfolio pf in items)
                {
                    if (_model.Portfolios.Contains(pf))
                    {
                        cnt++;
                    }
                }
            }

            // can execute if the count is > 0
            return cnt > 0;
        }
        /// <summary>
        /// Add a new item with the given symbol to the current portfolio.
        /// </summary>
        /// <param name="parameter"><see cref="SymbolName"/> that defines the asset to add.</param>
        public void Execute(object parameter)
        {
            var portfolios = parameter as IEnumerable;
            if (_model.PortfoliosView != null && portfolios != null)
            {
                // get current portfolio's position
                var pos = _model.PortfoliosView.CurrentPosition;

                // remove selected portfolios
                foreach (Portfolio pf in portfolios)
                {
                    if (_model.Portfolios.Count > 1)
                    {
                        _model.Portfolios.Remove(pf);
                    }
                }

                // restore selection
                _model.PortfoliosView.MoveCurrentToPosition(Math.Min(pos, _model.Portfolios.Count - 1));

                // command can no longer execute...
                OnCanExecuteChanged(EventArgs.Empty);
            }
        }
        /// <summary>
        /// Event that fires when the command enabled state should change.
        /// </summary>
        public event EventHandler CanExecuteChanged;
        /// <summary>
        /// Raises the <see cref="CanExecuteChanged"/> event.
        /// </summary>
        /// <param name="e"><see cref="EventArgs"/> that contains the event data.</param>
        protected virtual void OnCanExecuteChanged(EventArgs e)
        {
            if (CanExecuteChanged != null)
                CanExecuteChanged(this, e);
        }
    }
}
